#include "Terrain/ConstantBuffers.hlsl"
#include "Terrain/ClipmapCommonVS.hlsl"

VertexOut vertexFunction(VertexIn In)
{
  VertexOut Out = (VertexOut)0;

  float2 levelPos = In.Position.xy + patchOriginInClippedLevel;
  float2 clipmapPos = (levelPos + levelOffsetFromClipmapCenter) * levelScaleFactor * levelZeroWorldScaleFactor;
  float2 worldPos = clipmapPos * worldScale.xy + centerOffsetFromOrigin;

  float x = clamp(worldPos.x, worldExtents.x, worldExtents.z);
  float y = clamp(worldPos.y, worldExtents.y, worldExtents.w);

  float3 displacedPosition = float3(x, y, sampleHeight(Out, levelPos) * heightExaggeration) + worldOffset.xyz;

  Out.Position = mul(float4(displacedPosition, 1.0), worldToClipMatrix);
  Out.WorldPosition = float4(displacedPosition, 1.0);

  Out.ScreenPosition = Out.Position;

  Out.ClipDistance = applyClipping(float4(displacedPosition, 1.0) - worldOffset);

  return Out;
}
