#ifdef __cplusplus
#define cbuffer struct
#define shader_bool int
#define float4x4 mat4
#define float4 vec4
#define float2a vec2a
#define float2 vec2
#define int2 glm::ivec2
#define int4 glm::ivec4
#else
#define floata float
#define float2a float2
#define shader_bool bool
#endif


cbuffer GlobalParameters
{
  float4 worldExtents;
  float4 clipmapExtents;

  float2 levelZeroWorldScaleFactor;
  float2 centerOffsetFromOrigin;

  float2 unblendedRegionSize;
  float2 oneOverBlendedRegionSize;

  float4 worldScale;
  float4 worldOffset;

  float heightExaggeration;
  float noClipmapData;

  shader_bool enableNoData;
};

cbuffer LevelParameters
{
  // Per level uniform parameters
  float2 oneOverClipmapSize;
  float2 oneOverCoarseClipmapSize;

  float2 levelScaleFactor;
  float2 levelOffsetFromClipmapCenter;

  float2 fineLevelOriginInCoarse;
  float2 fineToCoarseRatio;

  float2 viewPosInClippedLevel;

  float2 fineTextureOrigin;

  shader_bool useBlendRegions;
  shader_bool useClipping;

  int levelIndex;
};

cbuffer ImageryParameters
{
  // Per level uniform imagery parameters
  float2a fineImageryOriginInCoarse[NUM_TEXTURES];

  float2a terrainToImageryResolutionRatio[NUM_TEXTURES];
  float2a fineToCoarseImageryResolutionRatio[NUM_TEXTURES];

  float2a terrainOffsetInImagery[NUM_TEXTURES];

  float2a oneOverImagerySize[NUM_TEXTURES];
  float2a oneOverCoarseImagerySize[NUM_TEXTURES];

  floata noData[NUM_TEXTURES];
};

cbuffer PatchParameters
{
  // Per patch uniform parameters
  float2 patchOriginInClippedLevel;
};

cbuffer RenderParameters
{
  float4 blendRegionColor;

  shader_bool showImagery;
  shader_bool showBlendRegions;

  shader_bool showLevelColors;
};

cbuffer LevelRenderParameters
{
  float4 color;
};

cbuffer OceanParameters
{
//  float4 oceanOrigin;
  float4 deepColor;
  float4 shallowColor;
  float4 roughColor;

  int4   tileOffset;
  float4 tileRemainder;
  float2 tileCoordsFromWorld;      // Scale world-space positions to base tile.

  float2 viewportScale;
  float2 lodDisplace;
  float2 lodNormal;
  float2 lodDetail;
  float2 lodFoam;


  float significantWaveHeightWorld;   // Significant wave height in world-space units.
  float worldSpaceMeter;
  float swellDirection;
  float foamDetectionScale;
  float foamErosionValue;       // def = 2.f
  float shoale;
  float shoalingEffectFactor;     // [0.0 -> 1.0], 0.0 disables geometric shoaling
  float roughness;
  float oneOverTileExtentAdjust;
};
